@@ -4,7 +4,15 @@ from copy import copy
 from functools import reduce, partial
 from operator import methodcaller, add, getitem, eq
 from typing import (
-    Mapping, Collection, Any, Union, Sequence, MutableMapping, Callable, Type
+    Mapping,
+    Collection,
+    Any,
+    Union,
+    Sequence,
+    MutableMapping,
+    Callable,
+    Type,
+    Hashable,
 )
 
 from cytoolz import merge
@@ -32,16 +40,25 @@ def to_records(nested: Mapping, accumulated_levels=None, level_names=None):
     return records
 
 
-def unnest(mapping_mapping):
-    unnested = []
-    for category, mapping in mapping_mapping.items():
-        unnested.append(
-            {
-                str(category) + "_" + str(key): value
-                for key, value in mapping.items()
-            }
-        )
-    return merge(unnested)
+def _unnest(nested, mtypes, escape, levels):
+    if not isinstance(nested, mtypes):
+        return nested
+    prefix = f"{escape}_{escape}".join(levels)
+    if len(prefix) > 0:
+        prefix = f"{escape}{prefix}{escape}_"
+    long_records = []
+    for level, maybe_mapping in nested.items():
+        if not isinstance(maybe_mapping, mtypes):
+            long_records.append({f"{prefix}{level}": maybe_mapping})
+            continue
+        long_records += _unnest(maybe_mapping, mtypes, escape,
+                                levels + [level])
+    return long_records
+
+
+def unnest(nested, mtypes=(dict,), escape=""):
+    flat_records = _unnest(nested, mtypes, escape, [])
+    return merge(flat_records)
 
 
 class NestingDict(defaultdict):
@@ -153,12 +170,14 @@ def constant_to_dig_predicate(putative_predicate, match="key", default=eq):
 def keyonly(func):
     def onlykey(key, _value):
         return func(key)
+
     return onlykey
 
 
 def valonly(func):
     def onlyval(_key, value):
         return func(value)
+
     return onlyval
 
 
@@ -247,7 +266,7 @@ def dig_and_edit(
     mapping: MutableMapping,
     filter_func: Callable[[Any, Any], Any],
     setter_func: Callable[[Any, Any], Any],
-    mtypes: tuple[Type[MutableMapping]] = (dict,)
+    mtypes: tuple[Type[MutableMapping]] = (dict,),
 ) -> MutableMapping:
     matches = tuple(filter(splat(filter_func), mapping.items()))
     for key, value in matches:
@@ -268,3 +287,28 @@ def separate_by(collection, ref):
         else:
             misses.append(item)
     return hits, misses
+
+
+class HashDict:
+    def __init__(self, equivalence: Callable[[Any], Hashable] = hash):
+        self.dict_ = {}
+        self.reverse = {}
+        self.hasher = equivalence
+
+    def __setitem__(self, key, item):
+        itemhash = self.hasher(item)
+        self.dict_[key] = itemhash
+        if itemhash not in self.reverse.keys():
+            self.reverse[itemhash] = item
+
+    def __getitem__(self, key):
+        return self.reverse[self.dict_[key]]
+
+    def __str__(self):
+        return (
+            f"HashDict with {len(self.dict_)} keys storing "
+            f"{len(self.reverse)} unique values"
+        )
+
+    def __repr__(self):
+        return self.__str__()
